#!/usr/bin/python
"""
make distrodb from apt-rpm pkglist.
"""
import os
import sys
import re
import DistroDbMaker
from DistroDbMaker.BinRPM2DistroDB import BinRPM2DistroDB
from DistroDbMaker.SrcRPM2DistroDB import SrcRPM2DistroDB
from DistroDbMaker import DispatcherRPM2DistroDB

class MixRepoReader(DistroDbMaker.RepoReader):

    def __init__(self, verbose=False):
        super(MixRepoReader,self).__init__(verbose)
        self.rpmheader_handler=DispatcherRPM2DistroDB(
            srchandler=SrcRPM2DistroDB(verbose),
            binhandler=BinRPM2DistroDB(verbose),
            verbose=verbose)
        self.src_repodata=[]
        self.bin_repodata=[]
        self.mix_repodata=[]

    def usage(self):
        super(MixRepoReader,self).usage()
        print '--src-repodata <dir>'
        print '--pkg-repodata <dir>'
        print '--mix-repodata <dir> TODO'

    def set_default_opt_handlers(self):
        super(MixRepoReader,self).set_default_opt_handlers()
        self.add_longopt("src-repodata=")
        self.set_opt_handler('--src-repodata', lambda reader,o,a: reader.src_repodata.append(a))
        self.add_longopt("pkg-repodata=")
        self.set_opt_handler('--pkg-repodata', lambda reader,o,a: reader.bin_repodata.append(a))

    def process_args(self, args):
        verbose=self.verbose
        src_primary_xml=[]
        bin_primary_xml=[]
        bin_filelists_xml=[]
        for repodatadir in self.src_repodata:
            if not os.path.isdir(repodatadir):
                print "not a dir: %s" % repodatadir
                sys.exit(1)
            for xml in DistroDbMaker.find_first_compressed('*primary.xml', repodatadir):
                src_primary_xml.append(xml)
        for repodatadir in self.bin_repodata:
            if not os.path.isdir(repodatadir):
                print "not a dir: %s" % repodatadir
                sys.exit(1)
            for xml in DistroDbMaker.find_first_compressed('*primary.xml', repodatadir):
                bin_primary_xml.append(xml)
            for xml in DistroDbMaker.find_first_compressed('*filelists.xml', repodatadir):
                bin_filelists_xml.append(xml)
        for repodatadir in self.mix_repodata:
            if not os.path.isdir(repodatadir):
                print "not a dir: %s" % repodatadir
                sys.exit(1)
            for xml in DistroDbMaker.find_first_compressed('*primary.xml', repodatadir):
                self.primary_xml.append(xml)
            for xml in DistroDbMaker.find_first_compressed('*filelists.xml', repodatadir):
                self.filelists_xml.append(xml)
        super(MixRepoReader,self).process_args(args)
        if len(src_primary_xml):
            for xml in src_primary_xml:
                print "processing src primary.xml repodata at %s..." % xml
                self.process_repodata_primary_xml(xml, self.rpmheader_handler.srchandler)
        if len(bin_primary_xml):
            for xml in bin_primary_xml:
                print "processing bin primary.xml repodata at %s..." % xml
                self.process_repodata_primary_xml(xml, self.rpmheader_handler.binhandler)
        if len(bin_filelists_xml) and self.process_repodata_filelists_xml:
            for xml in bin_filelists_xml:
                print "processing bin filelists.xml repodata at %s..." % xml
                self.process_repodata_filelists_xml(xml, self.rpmheader_handler.binhandler)

if __name__ == '__main__':
    reader=MixRepoReader()
    reader.run()
