#!/usr/bin/python
"""
make distrodb from apt-rpm pkglist.
"""
import sys
import re
import DistroDbMaker
import DistroDbMaker.Map

sourcename=DistroDbMaker.Map.MapOne2Many()
srcname2binnames=DistroDbMaker.Map.MapOne2Many()
provides=DistroDbMaker.Map.MapOne2Many()
requires=DistroDbMaker.Map.MapOne2Many()
perl=DistroDbMaker.Map.MapOne2Many()
pyegg=DistroDbMaker.Map.MapOne2Many()
python2raw=DistroDbMaker.Map.MapOne2Many()
python3raw=DistroDbMaker.Map.MapOne2Many()

# needs actual files
# buildreqs-subst
# groups-allowed

# TODO: headers-rebased

dirname_pattern=re.compile('^(.*)/[^/]*$')
def dirname(path):
    result=''
    match=dirname_pattern.match(path)
    if (match):
        result=match.group(1)
    return result

class FromPattern:
    """class for pattern db."""
    # TODO support for exclude_dirs
    def __init__(self, table, pattern,export=True,debug=False,exclude_dirs=False,transform=(lambda s: s)):
        self.table=table
        self.db = DistroDbMaker.Map.MapOne2Many()
        self.pattern = re.compile(pattern)
        self.export=export
        self.debug=debug
        self.exclude_dirs=exclude_dirs
        self.dirs={}
        self.transform=transform

    def set_db(self,db):
        self.db = db
        #print "set db to %s" % db
        return self

    def write(self,distrodb):
        self.postprocess()
        distrodb.writemap(self.table,self.db)

    def apply(self, path, name):
        match=self.pattern.match(path)
        if match:
            result=match.group(1)
            result=self.transform(result)
            #if self.lowercase:
            #    result=result.lower()
            if result != '' and result != False:
                self.db.add_pair(result, name)
                dir_to_exclude=result
                while self.exclude_dirs:
                    dir_to_exclude=dirname(dir_to_exclude)
                    self.dirs[dir_to_exclude]=1
                    if dir_to_exclude == '':
                        break
                if self.debug:
                    print "%s: %s match (result=%s)" % (name,path,result)
            return True
        else:
            if self.debug:
                print "%s: %s not match" % (name,path)
            return False

    def postprocess(self):
        db=self.db
        dirmap=self.dirs
        if self.exclude_dirs:
            for key in dirmap.keys():
                db.del_key(key)


x11_modules_pattern=re.compile('^X11/modules/')
def plugins_subst(raw):
    raw=x11_modules_pattern.sub('xorg/modules/',raw)
    return raw
mageia_multiarch_dir_pattern=re.compile('(^|/)multiarch-x86_64-linux(/|$)')
def kill_mageia_multiarch_dir(raw):
    raw=mageia_multiarch_dir_pattern.sub(lambda m: m.group(1) or m.group(2),raw)
    return raw
def cmake_filter_and_lower(raw):
    if raw=='Modules' or raw=='modules' or raw=='templates':
        raw='' # kill top dirs
    return raw.lower()
so_lib_kill_numbers_pattern=re.compile('\.so[\.\d]*$')
def so_lib_kill_numbers(raw):
    raw=so_lib_kill_numbers_pattern.sub('.so',raw)
    return raw
erlang_dir_version_pattern=re.compile('^([^/-]+)-[\d.]+/')
def erlang_kill_dir_version(raw):
    raw=erlang_dir_version_pattern.sub(lambda m: m.group(1)+'/',raw)
    return raw

independent_patterns=[];
independent_patterns.append(FromPattern('path','^(?:/usr)?(?:/games|/s?bin|(?:(?:/lib(?:64)?|/share)?(?:/kde[1-9]|/qt[1-9]|/kf5)/s?bin))/(.*)$',exclude_dirs=True,transform=kill_mageia_multiarch_dir,debug=False))

exclusive_patterns=[];
exclusive_patterns.append(FromPattern('mozplugin.raw','^/usr/lib64/(?:mozilla/plugins|browser-plugins)/(.*)$',exclude_dirs=True))
exclusive_patterns.append(FromPattern('shared-icons.noexport','^(/usr/share/icons/(?:(?:mini|large|hicolor|locolor|HighContrastInverse|LowContrast)/.*|[^/]*))$',export=False))
exclusive_patterns.append(FromPattern('shared-data.raw','^/usr/share/((?:doc/qt4|doc/qt5|qt4/translations|qt5/translations|gtk-doc/html|gnome-shell|mediawiki/extensions|hyphen|myspell|mythes|timidity|awesome/lib|thumbnailers|mate-panel/applets|glib-2.0/schemas|festival/dicts|festival/voices|doc/xorg-docs|gimp/2.0/scripts|dokuwiki/lib/plugins|themes|icons)/.*)$',exclude_dirs=True,export=True))
exclusive_patterns.append(FromPattern('aspell.raw','^/usr/lib64/aspell(?:-[\d\.]+)?/(.*)$',exclude_dirs=True))
exclusive_patterns.append(FromPattern('nagios.raw','^/usr/(?:lib|lib64|share)/nagios/plugins/(.*)$',exclude_dirs=True))
exclusive_patterns.append(FromPattern('vdr.raw','^/usr/lib64/vdr/(.*)$',exclude_dirs=True,transform=so_lib_kill_numbers))
exclusive_patterns.append(FromPattern('doc.noexport','^(/usr/share/(doc|man|info|javadoc)/.*)$',export=False))
exclusive_patterns.append(FromPattern('resources.noexport','^(/usr/share/(icons|pixmaps|locale|applnk|applications|application-registry|backgrounds|mime|mime-info|mimelnk|services|servicetypes)/.*)$',exclude_dirs=True,export=False))
exclusive_patterns.append(FromPattern('bin','^((?:/usr)?(?:/games|/s?bin|(?:(?:/lib(?:64)?|/share)?(?:/kde[1-9]|/qt[1-9]|/kf5)/s?bin))/.*)$',exclude_dirs=True,transform=kill_mageia_multiarch_dir))
exclusive_patterns.append(FromPattern('devel-libs','^(?:/usr)?/lib(?:64)?/lib(.*)\.so$'))
exclusive_patterns.append(FromPattern('static-libs','^(?:/usr)?/lib(?:64)?/lib(.*)\.a$'))
exclusive_patterns.append(FromPattern('headers','^/usr/include/(.*)$',exclude_dirs=True,transform=kill_mageia_multiarch_dir))
exclusive_patterns.append(FromPattern('pkg-config','^/usr/(?:lib(?:64)?|share)/pkgconfig/(.*)\.pc$'))
exclusive_patterns.append(FromPattern('cmake','^/usr/(?:lib(?:64)?|share)/cmake/([^/]*)/',exclude_dirs=True,transform=cmake_filter_and_lower))
exclusive_patterns.append(FromPattern('fonts.raw','^/usr/share/fonts/.*/([^/]*\.(?:ttc|ttf|ttx|otf|pcf|afm|pfb))(?:\.(?:gz|bz2|xz))?'))
#/usr/share/eog/gir-1.0/Eog-3.0.gir
# TODO: but conflict in mageia for /usr/share/org.gnome.Music/gir-1.0/Gd-1.0.gir 
# /usr/share/org.gnome.Documents/gir-1.0/Gd-1.0.gir
exclusive_patterns.append(FromPattern('gir','^/usr/share.*/gir-1\.0/(.*)\.gir$',debug=False))
exclusive_patterns.append(FromPattern('vapi','^/usr/share.*/vapi/(.*)\.vapi$',debug=False))
exclusive_patterns.append(FromPattern('texmf.raw','^/usr/share/(?:texmf|texmf-texlive|texmf-dist|texlive/texmf-dist)/(?!doc/)(.*)$',exclude_dirs=True))
exclusive_patterns.append(FromPattern('php.raw','^/usr/(?:lib|lib64|share)/(?:php/[^/]*/(?:extensions|modules)|php/(?:extensions|modules)|php/pear|php|pear)/(.*\..*)$',exclude_dirs=True,debug=False))
exclusive_patterns.append(FromPattern('java.raw','^/usr/(?:lib|lib64|share)/java/(.*\.jar)$'))
exclusive_patterns.append(FromPattern('mono.noexport','^/usr/lib/mono/gac/(.*)$',export=False))
exclusive_patterns.append(FromPattern('mono.raw','^/usr/lib/mono/(.*\.dll)$'))
exclusive_patterns.append(FromPattern('golang.raw','^/usr/(?:share/gocode|lib/golang)/src/(.*)$',exclude_dirs=True))
# mga: /usr/lib64/erlang/lib fc: /usr/lib64/erlang/lib /usr/share/erlang/lib alt: /usr/lib/erlang/lib
exclusive_patterns.append(FromPattern('erlang.raw','^/usr/(?:share|lib|lib64)/erlang/lib/(.*)$',exclude_dirs=True,transform=erlang_kill_dir_version))
# mga,alt,fc: /usr/lib64/ocaml
exclusive_patterns.append(FromPattern('ocaml.raw','^/usr/lib64/ocaml/(.*)$',exclude_dirs=True))
# mga,alt,fc: /usr/lib/node_modules
exclusive_patterns.append(FromPattern('nodejs.raw','^/usr/lib/node_modules/(.*)$'))

# mga,fc: /usr/i686-w64-mingw32/sys-root/mingw /usr/x86_64-w64-mingw32/sys-root/mingw
# alt: /usr/i586-pc-mingw32/sys-root/mingw /usr/x86_64-pc-mingw32/sys-root/mingw

# mga: /usr/share/gems/gems/ /usr/share/gems/specifications /usr/lib64/ruby/site_ruby /usr/share/ruby/site_ruby
# fc: /usr/lib64/ruby/vendor_ruby /usr/share/ruby/vendor_ruby /usr/share/gems/gems/ /usr/share/gems/specifications /usr/lib64/gems/ruby
# alt: /usr/lib/ruby/site_ruby

exclusive_patterns.append(FromPattern('shared-lib.raw','^/usr/lib/(cups/(?:backend|driver|filter)/.*)$',exclude_dirs=True))
#exclusive_patterns.append(FromPattern('',''))

# too vague and prone to collecting garbage:
# should be last or it will include perl,python,ruby, etc .so
exclusive_patterns.append(FromPattern('plugins.raw','^(?:/usr)?/lib(?:64)?/(gimp/2.0/plug-ins/.*|xfce4/panel-plugins/.*|.*\.so)$',transform=plugins_subst,exclude_dirs=True))
exclusive_patterns.append(FromPattern('unique_data.raw','^(/usr/share/.*)$').set_db(DistroDbMaker.Map.MapOne2Unique()))

# note: +? - lazy match not to eat (\.p[hlm])
perl_pattern=re.compile('^perl\(([^)]+?(\.p[hlm])?)\)$')
pyegg_pattern=re.compile('^(.*)/([^/]*\.egg-info)$')
pyegg_nover_pattern=re.compile('^([^-]*)(?:-.*)?\.egg-info$')
pyegg_ver_pattern=re.compile('^([^-]*)(?:-[^-]*)?-py([2-9]).*\.egg-info$')
jython_pattern=re.compile('^/usr/share/jython')
python_prefix_pattern=re.compile('^/usr/(?:lib(?:64)?|share)/python([0-9]+)(?:\.[0-9]+)?/')
python_pattern=re.compile('^/usr/(?:lib(?:64)?|share)/python([0-9]+)(?:\.[0-9]+)?/(?:site-packages/|lib-dynload/)?(.*\.(?:py|so))$')
sourcename_pattern=re.compile('^(.*)-[^-]+-[^-]+$')
#path_pattern=re.compile('/([^/]+)$')

def process_provides(providename,pkgname):
    provides.add_pair(providename,pkgname)
    result=perl_pattern.match(providename)
    if result:
        raw_perl=result.group(1)
        if result.group(2):
            # alt linux format
            perl.add_pair('perl('+raw_perl+')',pkgname)
        else:
            # fc format
            perl.add_pair('perl('+raw_perl.replace('::','/')+'.pm)',pkgname)

def process_requires(requirename,pkgname):
    if not requirename[:7] == 'rpmlib(' and not requirename == '/bin/sh':
        requires.add_pair(requirename,pkgname)

def process_package_file(f,name):
    for p in independent_patterns:
        p.apply(f,name)
    applied = False
    if not applied:
        #print result.group(1)
        result=python_pattern.match(f)
        if result:
            applied = True
            pyver=result.group(1)
            #pyinc=result.group(2).replace('/','.').replace('.__init__','')
            pyinc=result.group(2)
            if pyver == '2':
                python2raw.add_pair(pyinc,name)
            elif pyver == '3':
                #.cpython-35m-x86_64-linux-gnu.so -> .so
                pyinc=re.sub("\.cpython-3[0-9]*m(-[^-]*-linux-gnu)?\.so$",".so",pyinc)
                python3raw.add_pair(pyinc,name)
            else:
                print "invalid python: "+f
                sys.exit (1)
    if not applied:
        result=pyegg_pattern.match(f)
        #print result.group(2)
        if result:
            applied = True
            eggpath=result.group(1)
            eggstring=result.group(2)
            result=python_prefix_pattern.match(eggpath)
            if jython_pattern.match(eggpath):
                pass
            elif result:
                pyver=result.group(1)
                result=pyegg_nover_pattern.match(eggstring)
                if result:
                    eggname=result.group(1)
                    if eggname != "UNKNOWN":
                        pyegg.add_pair("pythonegg("+pyver+")("+eggname+")",name)
                else:
                    print "invalid pyegg1: "+f
                    sys.exit (1)
            else:
                result=pyegg_ver_pattern.match(eggstring)
                if result:
                    eggname=result.group(1)
                    pyver=result.group(2)
                    if eggname != "UNKNOWN":
                        pyegg.add_pair("pythonegg("+pyver+")("+eggname+")",name)
                else:
                    # ignore bad eggs silently
                    pass

    if not applied:
        for p in exclusive_patterns:
            if p.apply(f,name):
                applied = True
                #if debug:
                #    print "%s: matched %s" % (f,p.table)
                break

def python_unraw(raw):
    unraw=re.sub('\.(?:so|py)$','',raw).replace('/__init__','').replace('/__main__','').replace('/','.')
    return unraw

def process_rpm(name,sourcerpm,requirename,providename,filenames=None):
    if sourcerpm is None:
        print "Oops: undefined sourcerpm in %s :(" % name
        sys.exit(1)
    result=sourcename_pattern.match(sourcerpm)
    if result:
        srcname=result.group(1)
        sourcename.add_pair(name,srcname)
        srcname2binnames.add_pair(srcname,name)
    else:
        print "Oops: no match for name in sourcerpm %s :(" % sourcerpm
        sys.exit(1)
    if requirename:
        for p in requirename:
            # for compatibility with yum repodata
            if providename and p in providename:
                continue
            # optional: filenames array search is long,
            # so default is filenames=None
            if filenames and p in filenames:
                continue
            # END for compatibility with yum repodata
            process_requires(p,name)
    if providename:
        for p in providename:
            process_provides(p,name)

class BinRPM2DistroDB(DistroDbMaker.RPM2DistroDB):
    def __init__(self, verbose=False, ignore_srpms_in_binary=False):
        super(BinRPM2DistroDB,self).__init__(verbose)
        self.ignore_srpms_in_binary=ignore_srpms_in_binary
        #self.rpm5=False

    def mangle_name(self,name):
        # mageia specific; fedora safe
        if name[:5] == 'lib64':
            name='lib'+name[5:]
        # end mageia specific
        return name

    def arch_check(self,arch):
        if arch and (arch == 'noarch' or arch == 'x86_64'):
            return True
        return False

    def process_rpm_header(self,rpmheader,isSource=False, filepath=None):
        if isSource:
            if self.ignore_srpms_in_binary:
                return
            else:
                raise ValueError("Oops! header for %s is from a source package" % rpmheader['NAME'])
        #print '-'.join(rpmheader[x] for x in ('name', 'version', 'release'))
        if self.arch_check(rpmheader['ARCH']):
            name=self.mangle_name(rpmheader['NAME'])
            if self.name_filter and not name in self.name_filter:
                return
            sourcerpm=rpmheader['SOURCERPM']
            if sourcerpm is None:
                if self.ignore_srpms_in_binary:
                    return
                else:
                    raise ValueError("Oops! header for %s has no SOURCERPM (from a source package?)" % rpmheader['NAME'])
            process_rpm(name,sourcerpm,rpmheader['REQUIRENAME'],rpmheader['PROVIDENAME'])
            self.process_filelist(rpmheader['FILENAMES'],name)


    def process_filelist(self,filenames,name):
        if not filenames:
            if self.debug:
                print "info: %s has no filenames" % name
            return
        # alt and SuSE debuginfo packages
        if name[-10:] == '-debuginfo':
            return
        for f in filenames:
            process_package_file(f,name)
 
    def write_distrodb(self,distrodb):
        # postprocess hacks
        #cmakeEPL=filter(lambda x: x.table == 'cmake', exclusive_patterns)
        #cmakedb=cmakeEPL[0].db.map
        #if 'Modules' in cmakedb: del cmakedb['Modules']
        #if 'modules' in cmakedb: del cmakedb['modules']
        python2=DistroDbMaker.Map.MapOne2Many()
        python3=DistroDbMaker.Map.MapOne2Many()
        python2rawmap=python2raw.map
        python3rawmap=python3raw.map
        for p in python2rawmap:
            unraw=python_unraw(p)
            for q in python2rawmap[p]:
                python2.add_pair(unraw,q);
        for p in python3rawmap:
            unraw=python_unraw(p)
            for q in python3rawmap[p]:
                python3.add_pair(unraw,q);
        # end postprocess hacks

        for p in independent_patterns:
            if p.export:
                p.write(distrodb)
        for p in exclusive_patterns:
            if p.export:
                p.write(distrodb)
        distrodb.writemap('provides',provides)
        distrodb.writemap('requires.raw',requires)
        distrodb.writemap('pyegg.raw',pyegg)
        distrodb.writemap('python2',python2)
        distrodb.writemap('python3',python3)
        distrodb.writemap('python2.raw',python2raw)
        distrodb.writemap('python3.raw',python3raw)
        distrodb.writemap('perl',perl)
        distrodb.writemap('sourcename',sourcename)
        distrodb.writemap('srcname2binnames.raw',srcname2binnames)
        distrodb.write_version()
