"""
parse yum xmls.
"""
import sys
import re
#from gzip import GzipFile
from DistroDbMaker.decompress import DecompressedTmpFile

import xml.etree.cElementTree as ET
#try:
#    import xml.etree.cElementTree as ET
#except ImportError:
#    import xml.etree.ElementTree as ET

#filelists_xml_files_pattern=re.compile('^ *<file(?:>|[^>]*?( type="dir")?[^>]*>)(.*)</file>$')
filelists_xml_files_pattern=re.compile('^ *<file[^>]*>(.*)</file>$')
filelists_xml_package_pattern=re.compile('^ *<package.* name="([^"]+)" arch="([^"]+)".*$')
filelists_xml_ignore_pattern=re.compile('^\s*<(?:filelists|/filelists|/package|version|\?xml).*$')
filelists_xml_empty_pattern=re.compile('^\s*$')

def unicode2str(ustr):
    if ustr==None:
        return ustr
    return ustr.encode('utf-8')

def xml_collect_rpm_entry(child):
    retlist=[]
    for entry in child:
        retlist.append(entry.attrib['name'].encode('utf-8'))
    return retlist

def process_ET_xml_package(rpmheader_handler,package):
    header={}
    header['FILENAMES']=None
    arch=None
    location=None
    for child in package:
        if child.tag[-5:] == '}name':
            header['NAME']=child.text.encode('utf-8')
        elif child.tag[-8:] == '}version':
            for attr in child.attrib:
                #print "version: attr=%s" % attr
                if attr[-4:] == 'ver':
                    header['VERSION']=child.attrib[attr].encode('utf-8')
        elif child.tag[-5:] == '}arch':
            arch=child.text.encode('utf-8')
            header['ARCH']=arch
        elif child.tag[-4:] == '}url':
            header['URL']=unicode2str(child.text)
        elif child.tag[-9:] == '}location':
            location=child.attrib['href'].encode('utf-8')
        elif child.tag[-7:] == '}format':
            provides=[]
            requires=[]
            for child2 in child:
                # {http://linux.duke.edu/metadata/rpm}sourcerpm
                if child2.tag[-13:] == 'rpm}sourcerpm':
                    header['SOURCERPM']=unicode2str(child2.text)
                elif child2.tag[-12:] == 'rpm}provides':
                    provides=xml_collect_rpm_entry(child2)
                elif child2.tag[-12:] == 'rpm}requires':
                    requires=xml_collect_rpm_entry(child2)
                #else:
                #    print "ignored child2=%s" % child2.tag
            header['PROVIDENAME']=provides
            header['REQUIRENAME']=requires
        #else:
        #    print "ignored tag=%s" % child.tag
    #print header
    rpmheader_handler.process_rpm_header(header, arch=='src',location)

# memory-spending fastest implementation
def process_repodata_primary_xml_parse(xmlfile, rpmheader_handler):
    tmpfile=DecompressedTmpFile(xmlfile)
    tree = ET.ElementTree(file=tmpfile.get_tmpfile_path())
    tmpfile.cleanup_tmpfile()
    #for package in tree.iter(tag='package'):
    root = tree.getroot()
    for package in root:
        #print "tag=%s" % package.tag
        # {http://linux.duke.edu/metadata/common}package
        assert(package.tag[-7:] == 'package')
        process_ET_xml_package(rpmheader_handler,package)

# memory-conserving fast implementation - for Large files
def process_repodata_primary_xml_iter3(xmlfile, rpmheader_handler):
    tmpfile=DecompressedTmpFile(xmlfile)
    context=ET.iterparse(tmpfile.get_tmpfile_path(), events=("end",))
    tmpfile.cleanup_tmpfile()
    for event, elem in context:
        if elem.tag[-7:] == 'package':
            process_ET_xml_package(rpmheader_handler,elem)
            elem.clear()

# memory-conserving implementation - also cleans root for XUGE files
def process_repodata_primary_xml_iter2(xmlfile, rpmheader_handler):
    tmpfile=DecompressedTmpFile(xmlfile)
    # get an iterable
    context = ET.iterparse(tmpfile.get_tmpfile_path(), events=("end",))
    tmpfile.cleanup_tmpfile()
    # turn it into an iterator
    context = iter(context)
    # get the root element
    event, root = context.next()
    counter=0
    for event, elem in context:
        if elem.tag[-7:] == 'package':
            process_ET_xml_package(rpmheader_handler,elem)
            counter += 1
            if counter % 10000 == 0:
                root.clear()

# do not use:
# memory-conserving slowest implementation - maybe due to extra start events
def process_repodata_primary_xml_iter1(xmlfile, rpmheader_handler):
    tmpfile=decompress.DecompressedTmpFile(xmlfile)
    header=None
    list_collector=None
    # get an iterable
    context = ET.iterparse(tmpfile.get_tmpfile_path(), events=("start", "end"))
    tmpfile.cleanup_tmpfile()
    # see http://effbot.org/zone/element-iterparse.htm#incremental-parsing
    # turn it into an iterator
    context = iter(context)
    # get the root element
    event, root = context.next()
    for event, elem in context:
        if event == "start":
            if elem.tag[-7:] == 'package':
                if header==None:
                    header={}
                    header['FILENAMES']=[]
                    header['PROVIDENAME']=[]
                    header['REQUIRENAME']=[]
                else:
                    print "Oops! non-empty header %s" % header
                    sys.exit(1)
            elif elem.tag[-12:] == 'rpm}provides':
                if list_collector == None:
                    list_collector=[]
                else:
                    print "Oops! non-empty collector - provides"
                    sys.exit(1)
            elif elem.tag[-12:] == 'rpm}requires':
                if list_collector == None:
                    list_collector=[]
                else:
                    print "Oops! non-empty collector - requires"
                    sys.exit(1)
        else: # event == "end":
            if elem.tag[-5:] == '}name':
                header['NAME']=elem.text.encode('utf-8')
            elif elem.tag[-8:] == '}version':
                for attr in elem.attrib:
                    #print "version: attr=%s" % attr
                    if attr[-4:] == 'ver':
                        header['VERSION']=elem.attrib[attr].encode('utf-8')
                header['VERSION']=elem.text.encode('utf-8')
            elif elem.tag[-5:] == '}arch':
                header['ARCH']=elem.text.encode('utf-8')
            elif elem.tag[-4:] == '}url':
                header['URL']=unicode2str(elem.text)
            elif elem.tag[-13:] == 'rpm}sourcerpm':
                header['SOURCERPM']=unicode2str(elem.text)
            # check list_collector because entry can be conflicts, obsoletes, etc.
            elif elem.tag[-9:] == 'rpm}entry' and list_collector!=None:
                list_collector.append(elem.attrib['name'].encode('utf-8'))
            elif elem.tag[-12:] == 'rpm}provides':
                header['PROVIDENAME']=list_collector
                list_collector=None
            elif elem.tag[-12:] == 'rpm}requires':
                header['REQUIRENAME']=list_collector
                list_collector=None
            elif elem.tag[-7:] == 'package':
                #print "debug: header %s" % header
                rpmheader_handler.process_rpm_header(header, header['ARCH']=='src')
                header=None
                root.clear()
            #elem.clear()

def process_repodata_filelists_xml(xmlfile, rpmheader_handler):
    tmpfile=DecompressedTmpFile(xmlfile)
    opened_tmp=open(tmpfile.get_tmpfile_path())
    tmpfile.cleanup_tmpfile()
    name=None
    arch=None
    filelist=[]
    try:
        for line in opened_tmp:
            if filelists_xml_ignore_pattern.match(line) or filelists_xml_empty_pattern.match(line):
                pass
            else:
                match=filelists_xml_files_pattern.match(line)
                if match:
                    filelist.append(match.group(1).replace('&amp;','&').replace('&gt;','>').replace('&lt;','<'))
                else:
                    match=filelists_xml_package_pattern.match(line)
                    if match:
                        if name:
                            rpmheader_handler.process_filelist_xml(filelist, name, arch, arch=='src')
                        name=match.group(1)
                        arch=match.group(2)
                        filelist=[]
                    else:
                        print "repodata_filelists_xml: %s: unexpected xml line: %s" % (xmlfile,line)
                        sys.exit(1)
        rpmheader_handler.process_filelist_xml(filelist, name, arch, arch=='src')
    finally:
        opened_tmp.close()

