"""
uncompress to a tmp file.
"""
import os
import tempfile
import bz2, gzip
# NOTE: use lzma for python3 (not in python 2.7)
import backports.lzma
import subprocess

def supported_suffixes():
    return ['','.gz','.bz2','.xz','.zck','.cz']

def decompress(fh):
    tmp_fd, tmp_name = tempfile.mkstemp(prefix='distrodb', suffix='.raw')
    os.write(tmp_fd, fh.read())
    os.fsync(tmp_fd)
    return tmp_name

def get_decompressed_file(compressed_name):
    istemp = False
    if compressed_name.endswith('.bz2'):
        fh = bz2.BZ2File(compressed_name)
        filename = decompress(fh)
        istemp = True
        fh.close()
    elif compressed_name.endswith('.xz'):
        fh = backports.lzma.open(compressed_name)
        filename = decompress(fh)
        istemp = True
        fh.close()
    elif compressed_name.endswith('.gz'):
        fh = gzip.GzipFile(compressed_name)
        filename = decompress(fh)
        istemp = True
        fh.close()
    elif compressed_name.endswith('.cz'):
        #see genhdlist in urpmi (rosa/mageia) for details;
        # not a proper gzip file, raise IOError, 'Not a gzipped file'
        # but hack below seem to work with the message
        # gunzip: hdlist.gz: decompression OK, trailing garbage ignored
        tmp_fd, filename = tempfile.mkstemp(prefix='distrodb', suffix='.raw')
        try:
            subprocess.check_call(['gunzip', '-S','.cz', '-c', compressed_name],stdout=tmp_fd)
        except subprocess.CalledProcessError as e:
            # exit code 2 expected - not a pure gz
            # gunzip: hdlist.cz: decompression OK, trailing garbage ignored
            pass
        istemp = True
    elif compressed_name.endswith('.zck'):
        tmp_fd, filename = tempfile.mkstemp(prefix='distrodb', suffix='.raw')
        try:
            subprocess.check_call(['unzck', '-c', compressed_name],stdout=tmp_fd)
        except subprocess.CalledProcessError as e:
            raise ValueError("Oops! %s is not a valid zchunk file" % compressed_name)
        istemp = True
    else:
        filename = compressed_name
    return istemp, filename

class DecompressedTmpFile(object):
    """
    an object for a tmp file.
    """
    def __init__(self, compressed_file):
        self.srpm_filter=None
        self.istemp, self.filename = get_decompressed_file(compressed_file)

    def get_tmpfile_path(self):
        return self.filename

    def cleanup_tmpfile(self):
        if self.istemp:
            try:
                os.unlink(self.filename)
            except OSError:
                pass
